#include "CivProjSimulation.h"
#include "CivCommonUtils.h"
#include "epanet2.h"
#include "CivInpConvertor.h"
#include "CivHydrTableHelper.h"

CivProjSimulation::CivProjSimulation(const std::string& uri):mUri(uri)
{
    mRptFile = CivCommonUtils::getExePath() + "\\test.rpt";
    mBinFile = CivCommonUtils::getExePath() + "\\test.bin";
}

CivProjSimulation::~CivProjSimulation()
{

}

bool CivProjSimulation::simulationProj(const std::string& projCode, const std::string& time)
{
    mSimulTime = time;
    mProjCode = projCode;

    // תinp֮ǰά
    CivHydrTableHelper helper(mUri);
    if (!helper.mainTain())
        return false;

    // codeӳ
    mProjCache.getCodeToSnMap(mUri);

    // תͨinpļ
    CivInpConvertor conventor(mUri);
    std::string baseInp = conventor.convertNotPorjInp(time);

    // תinpļ
   std::string projInp =  conventor.convertPorjInp(projCode, time);
    
   char baseInpFile[256];
   strcpy(baseInpFile, baseInp.c_str());

   char projInpFile[256];
   strcpy(projInpFile, projInp.c_str());

   if (!projBeforeHydrCompute(baseInpFile))
       return false;

   if (!projHydrCompute(projInpFile))
       return false;

   if (!saveToDb())
       return false;

	return true;
}

bool CivProjSimulation::saveToDb()
{
	return mProjCache.saveToUri(mUri,mSimulTime,mProjCode);
}

bool CivProjSimulation::projBeforeHydrCompute(char* inp)
{
    short hydCode = 1;
    long  t(0), tstep(0);
    int   iTime(0);

    int  errcode = 0;

    char rptFile[128];
    strcpy_s(rptFile, mRptFile.c_str());

    char binFile[128];
    strcpy_s(binFile, mBinFile.c_str());

    errcode = ENopen(inp, rptFile, binFile);

    if (errcode > 0)
    {
        ENclose();
        return false;
    }

    if (ENopenH() > 0)
    {
        ENclose();
        return false;
    }

    if (ENinitH(0) > 0)
    {
        ENclose();
        return false;
    }
    float dHStep;
    ENGetVal(4, &dHStep);
    do
    {
        int nLinkCount;
        ENgetcount(EN_LINKCOUNT, &nLinkCount);
        /*for (int i = 1; i <= nLinkCount; i++)
        {
            int linkType;
            char id[128] = "";
            ENgetlinktype(i, &linkType);
            ENgetlinkid(i, id);
        }*/

        if (ENrunH(&t) > 100)	// errcode > 100 Ǵ
        {
            ENclose();
            return false;
        }
        // ȡ
        getNodeResult();
        getLinkResult();

        ENnextH(&tstep); 
        iTime++;
    } while (tstep > 0);

    ENcloseH();
    ENclose();

    return true;
}

bool CivProjSimulation::projHydrCompute(char* inp)
{
    short hydCode = 1;
    long  t(0), tstep(0);
    int   iTime(0);

    int  errcode = 0;

    char rptFile[128];
    strcpy_s(rptFile, mRptFile.c_str());

    char binFile[128];
    strcpy_s(binFile, mBinFile.c_str());

    errcode = ENopen(inp, rptFile, binFile);
    if (errcode > 0)
    {
        ENclose();
        return false;
    }

    if (ENopenH() > 0)
    {
        ENclose();
        return false;
    }

    if (ENinitH(0) > 0)
    {
        ENclose();
        return false;
    }
    float dHStep;
    ENGetVal(4, &dHStep);

    do
    {
        int nLinkCount;
        ENgetcount(EN_LINKCOUNT, &nLinkCount);
        /*for (int i = 1; i <= nLinkCount; i++)
        {
            int linkType;
            char id[128] = "";
            ENgetlinktype(i, &linkType);
            ENgetlinkid(i, id);
        }*/

        if (ENrunH(&t) > 100)	// errcode > 100 Ǵ
        {
            ENclose();
            return false;
        }

        // 洢ģ
        getProjNodeResult();
        getProjLinkResult();

        ENnextH(&tstep);
        iTime++;
    } while (tstep > 0);

    ENcloseH();
    ENclose();

    return true;
}

void CivProjSimulation::getProjNodeResult()
{
    // ڵ
    int nNodeCount;
    ENgetcount(EN_NODECOUNT, &nNodeCount);
    int* typeCode = (int*)malloc(sizeof(int));

    for (int i = 1; i <= nNodeCount; i++)
    {
        ENgetnodetype(i, typeCode);
        if (*typeCode != EN_JUNCTION)
            continue;

        NodeResultItem nodeItem;

        ENgetnodeid(i, nodeItem.szNo);//        
        ENgetnodevalue(i, EN_HEAD, &nodeItem.dHead);// ˮͷ 
        mProjCache.addProjNodeHead(nodeItem.szNo, nodeItem.dHead);

        ENgetnodevalue(i, EN_PRESSURE, &nodeItem.dPressure); // ѹ 
        mProjCache.addProjNodePressure(nodeItem.szNo, nodeItem.dPressure);
    }
    free(typeCode);
}

void CivProjSimulation::getProjLinkResult()
{
    int  nLinkCount;
    ENgetcount(EN_LINKCOUNT, &nLinkCount);
    int* typeCode = (int*)malloc(sizeof(int));
  
    for (int i = 1; i <= nLinkCount; i++)
    {
        LinkResultItem linkItem;
        ENgetlinktype(i, typeCode);
        if (*typeCode != EN_PIPE)
            continue;

        ENgetlinkid(i, linkItem.szNo);  //    
        ENgetlinkvalue(i, EN_FLOW, &linkItem.dFlow);//      

        mProjCache.addProjPipeFlow(linkItem.szNo, linkItem.dFlow);

        ENgetlinkvalue(i, EN_VELOCITY, &linkItem.dVelocity);//    
        mProjCache.addProjPipeVelcoity(linkItem.szNo, linkItem.dVelocity);

        ENgetlinkvalue(i, EN_HEADLOSS, &linkItem.dHeadloss);// ˮͷʧ
        mProjCache.addPipeHeadLoss(linkItem.szNo, linkItem.dHeadloss);

    }
    free(typeCode);
}

void CivProjSimulation::getNodeResult()
{
    // ڵ
    int nNodeCount;
    ENgetcount(EN_NODECOUNT, &nNodeCount);
    int* typeCode = (int*)malloc(sizeof(int));

    for (int i = 1; i <= nNodeCount; i++)
    {
        ENgetnodetype(i, typeCode);
        if (*typeCode != EN_JUNCTION)
            continue;

        ProjNodeResultItem nodeItem;

        ENgetnodeid(i, nodeItem.szNo);//        
        ENgetnodevalue(i, EN_HEAD, &nodeItem.dHead);// ˮͷ  
        ENgetnodevalue(i, EN_PRESSURE, &nodeItem.dPressure); // ѹ 

        nodeItem.dDateTime =CivCommonUtils::currentTime(); // ǰʱ
        nodeItem.dTime = mSimulTime;
        nodeItem.projCode = mProjCode;
        mProjCache.addNodeItem(nodeItem);
    }
    free(typeCode);
}

void CivProjSimulation::getLinkResult()
{
    int  nLinkCount;
    ENgetcount(EN_LINKCOUNT, &nLinkCount);
    int* typeCode = (int*)malloc(sizeof(int));

    for (int i = 1; i <= nLinkCount; i++)
    {
        PorjPipeResultItem linkItem;
        ENgetlinktype(i, typeCode);
        if (*typeCode != EN_PIPE)
            continue;

        ENgetlinkid(i, linkItem.szNo);  //    
        ENgetlinkvalue(i, EN_FLOW, &linkItem.dFlow);//      
        ENgetlinkvalue(i, EN_VELOCITY, &linkItem.dVelocity);//    
        ENgetlinkvalue(i, EN_HEADLOSS, &linkItem.dHeadLoss);// ˮͷʧ

        linkItem.dDateTime = CivCommonUtils::currentTime(); // ǰʱ
        linkItem.dTime =mSimulTime ;
        linkItem.projCode = mProjCode;
        mProjCache.addPipeItem(linkItem);
    }
    free(typeCode);
}