#ifndef CIVASSEMBLY_H
#define CIVASSEMBLY_H
#include "CivTypes.h"
#include <list>
#include "pandaDbManager.h"

#pragma warning(disable:4251)

#define Str std::string 

class CivAssembly;
class CivNode;
class CivPipe;
class CivTank;
class CivValve;
class CivPumps;
class CivEmitters;
class CivCoordinates;
class CivMixing;
class CivLabels;
class CivTags;

/**
	 
*/
class CivAssembly
{
};

/**
   ڵ
 */
class PANDADBMANAGER_API CivNode: public CivAssembly
{
public:
	typedef struct  Table
	{
		// ; ID   Elev  Demand  	Pattern
		Str ID;
		Str Elev;
		Str Demand;
		Str PatternId;
	}  NodeTable;

	void addItem(const NodeTable& table);
	std::list<NodeTable> mTables;
};

/**
 * 
 */
class PANDADBMANAGER_API CivPipe: public CivAssembly
{
public:
	typedef struct  Table
	{
		// ; ID  Node1   Node2   Length   Diameter  	Roughness  MinorLoss  Status
		Str ID;
		Str Node1;
		Str Node2;
		Str Length;
		Str Diameter;
		Str Roughness;
		Str MinorLoss;
		Str Status;
		Str code;
	}  PipesTable;

	void addItem(const PipesTable& table);
	std::list<PipesTable> mTables;
};


/*
ˮ
*/
class PANDADBMANAGER_API CivTank:public CivAssembly
{
public:
	typedef struct  Table
	{
		// ; ID   Elevation  InitLevel  MinLevel   MaxLevel   Diameter   MinVol   VolCurve
		Str ID;
		Str Elev;
		Str InitLevel;
		Str MinLevel;
		Str MaxLevel;
		Str Diameter;
		Str MinVol;
		Str VolCurve;
	}  TankTable;

	void addItem(const TankTable& table);
	std::list<TankTable> mTables;
};

/*
 ڹпƷš
*/
class PANDADBMANAGER_API CivValve :public CivAssembly
{
public:
	typedef struct  Table
	{
		Str ID;
		Str Node1;
		Str Node2;
		Str Diameter;
		Str Type;
		Str Setting;
		Str MinorLoss;
	}  ValveTable;

	void addItem(const ValveTable& table);

	std::list<ValveTable> mTables;
};

/**
  ˮ
*/
class PANDADBMANAGER_API CivPumps :public CivAssembly
{
public:
	typedef struct  Table
	{
		// ;ID   Node1   Node2   Parameters
		Str ID;
		Str Node1;
		Str Node2;
		Str Parameters = "HEAD 1";
	}  PumpTable;

	void addItem(const PumpTable& table);
	std::list<PumpTable> mTables;
};

/*
аˮڵ
*/
class PANDADBMANAGER_API CivReservoirs :public CivAssembly
{
public:
	typedef struct  Table
	{
		/* 
			IDǩ
			ˮͷft(m)
			ˮͷģʽIDѡ
		*/
		Str ID;
		Str Head;
		Str Pattern;
	}  ReservoirsTable;

	void addItem(const ReservoirsTable& table);

	std::list<ReservoirsTable> mTables;
};

/**
   EMITTERSģڵ㶨Ϊɢ߿׿ڣ
*/
class PANDADBMANAGER_API CivEmitters :public CivAssembly
{
public:
	typedef struct  Table
	{
		/*
			ӽڵIDǩ
			ϵ1ף1 psiѹµλ
		*/
		Str ID;
		Str coeff;
	}  EmittersTable;

	void addItem(const EmittersTable& table);
	std::list<EmittersTable> mTables;
};

/**
  
*/
class PANDADBMANAGER_API CivCoordinates :public CivAssembly
{
public:
	typedef struct  Table
	{
		Str ID;
		Str XCoord;
		Str YCoord;
		Str code;
	}  CoordTable;

	

	void addItem(const CoordTable& table);
	std::list<CoordTable> mTables;
};

/**
	ѡ
*/
class PANDADBMANAGER_API  CivMixing :public CivAssembly
{
public:
	typedef struct  Table
	{
		/*
			ˮIDǩ
			ģͣMIXED, 2COMP, FIFOLIFO
			ҵݻС
		*/
		Str ID;
		Str hyModel;
		Str volume;
	}  MixingTable;

	void addItem(const MixingTable& table);
	std::list<MixingTable> mTables;
};

/**
   ȷˮػϵģ͡
*/
class PANDADBMANAGER_API  CivParameter :public CivAssembly
{
public:
	typedef struct  Table
	{
		/*
			[OPTIONS]
			[TIMES]
			[REACTIONS]
			[ENERGY] 
			[REPORT]
		*/
		Str name;
		Str val;

		bool operator==(const Table& oterhtable)
		{
			return name == oterhtable.name;
		}
	} ParamTable;

	void addItem(const ParamTable& table);
	void setType(const std::string& type) { mType = type; }

	void merge(const CivParameter& otherParam)
	{
		if (mType != otherParam.mType)
			return;

		std::list<ParamTable> tables = otherParam.mTables;
		for (auto iter = tables.begin(); iter != tables.end(); iter++)
		{
			std::string name = iter->name;
			std::list<ParamTable>::iterator findIter = std::find(mTables.begin(), mTables.end(),*iter);
			if (findIter == mTables.end())
			{
				mTables.push_back({ iter->name,iter->val });
				continue;
			}
			 findIter->val =iter->val;
		}

	}

	std::string mType;
	std::list<ParamTable> mTables;
};

/**
  ģʽ
*/
class PANDADBMANAGER_API CivPatterns :public CivAssembly
{
public:
	typedef struct  Table
	{
		Str name; // ģʽ
		Str val; // ģʽֵ
	} PatternTable;

	void addItem(const PatternTable& table);
	std::list<PatternTable> mTables;
};

/*
	߼XY㡣
*/
class PANDADBMANAGER_API CivCurves :public CivAssembly
{
public:
	typedef struct  Table
	{
		Str ID; // ģʽ
		Str XCoord; // Xֵ
		Str YCoord; // Yֵ
	} CurvesTable;

	void addItem(const CurvesTable& table);
	std::list<CurvesTable> mTables;
};

/**
	ӽڵĶģʽˮǶ[JUNCTIONS]Ĳ䡣
*/
class PANDADBMANAGER_API CivDemands :public CivAssembly
{
public:
	typedef struct  Table
	{
		/* 
			ӽڵIDǩ
		    ˮλ
			ˮģʽIDѡ
			ֺ֮ˮƣѡ
		*/
		Str ID; 
		Str BaseDemand; 
		Str PatternId; 
		Str DemandType;
	} DemandsTable;

	void addItem(const DemandsTable& table);
	std::list<DemandsTable> mTables;
};

// ˮԴͷλá
class PANDADBMANAGER_API  CivSources :public CivAssembly
{
public:
	typedef struct  Table
	{
		/*
		ڵIDǩ
        ԴͷͣCONCEN, MASS, FLOWPACEDSETPOINT
        ׼Դͷǿ
        ʱģʽIDѡ;Node  Type  Quality  Pattern
		*/
		Str ID;
		Str Type;
		Str Quality;
		Str Pattern;
	} CurvesTable;

	void addItem(const CurvesTable& table);
	std::list<CurvesTable> mTables;
};

// ģ⿪ʼʱѡܶεĳʼ״̬
class PANDADBMANAGER_API CivStatus :public CivAssembly
{
public:
	typedef struct  Table
	{
		/*
		   ܶIDǩ
           ״̬ ID    Status/Setting
		*/
		Str ID;
		Str Setting;
	} StatusTable;

	void addItem(const StatusTable& table);
	std::list<StatusTable> mTables;
};

// ڵĳʼˮʡ
class PANDADBMANAGER_API CivQuality :public CivAssembly
{
public:
	typedef struct  Table
	{
		/*
		    ڵIDǩ
            ʼˮ
		*/
		Str ID;
		Str InitQuality;
	} QualityTable;

	void addItem(const QualityTable& table);
	std::list<QualityTable> mTables;
};

// ͼǩֵ
class  PANDADBMANAGER_API CivLabels :public CivAssembly
{
public:
	typedef struct  Table
	{
		/*
		  X
		  Y
		  ˫еıǩı
		  êڵIDǩѡ
		*/
		Str XCoord;
		Str YCoord;
		Str Label;
		Str ID;
	} LabelTable;

	void addItem(const LabelTable& table);

	std::list<LabelTable> mTables;
};

// ͱǩضڵ͹ܶ
class PANDADBMANAGER_API CivTags :public CivAssembly
{
public:
	typedef struct  Table
	{
		/*
		   ؼΪNODELINK
		   ڵ߹ܶIDǩ
		   ǩı޿ո
		*/
		Str Key;
		Str Id;
		Str Label;
	} TagsTable;

	void addItem(const TagsTable& table);
	std::list<TagsTable> mTables;
};

#endif // !CIVASSEMBLY_H

